<?php

namespace Objectiv\Plugins\Checkout\Model;

class Bump {
	protected $id;
	protected $title;
	protected $display_for;
	protected $products;
	protected $any_product = false;
	protected $location;
	protected $discount_type;
	protected $offer_product;
	protected $offer_discount;
	protected $offer_language;
	protected $offer_description;
	protected $upsell = false;

	public function __construct() {}

	/**
	 * @param int $id
	 * @throws \Exception
	 */
	public function load( int $id ): bool {
		$bump = get_post( $id );

		if ( empty( $bump ) ) {
			return false;
		}

		$this->id                = $id;
		$this->title             = $bump->post_title;
		$this->display_for       = get_post_meta( $this->id, 'cfw_ob_display_for', true );
		$this->products          = get_post_meta( $this->id, 'cfw_ob_products', true );
		$this->any_product       = get_post_meta( $this->id, 'cfw_ob_any_product', true ) === 'yes';
		$this->location          = get_post_meta( $this->id, 'cfw_ob_display_location', true );
		$this->discount_type     = get_post_meta( $this->id, 'cfw_ob_discount_type', true );
		$this->offer_product     = get_post_meta( $this->id, 'cfw_ob_offer_product', true );
		$this->offer_discount    = get_post_meta( $this->id, 'cfw_ob_offer_discount', true );
		$this->offer_language    = get_post_meta( $this->id, 'cfw_ob_offer_language', true );
		$this->offer_description = get_post_meta( $this->id, 'cfw_ob_offer_description', true );
		$this->upsell            = get_post_meta( $this->id, 'cfw_ob_upsell', true ) === 'yes';

		return true;
	}

	/**
	 * @return bool
	 */
	public function is_displayable(): bool {
		$offer_product = $this->get_offer_product();

		if ( ! $offer_product->is_in_stock() && ! $offer_product->backorders_allowed() ) {
			return false;
		}

		if ( $this->quantity_in_cart( $this->offer_product ) ) {
			return false;
		}

		if ( $this->is_valid_upsell() && $this->quantity_in_cart( array_values( $this->products )[0] ) > $offer_product->get_stock_quantity() ) {
			return false;
		}

		if ( 'all_products' === $this->display_for ) {
			return true;
		}

		$matching_products_in_cart = 0;

		// Count matching products in the cart
		foreach ( $this->products as $product ) {
			if ( $this->quantity_in_cart( (int) $product ) ) {
				$matching_products_in_cart++;
			}
		}

		// If all products must match and we have fewer products in the cart than in our matching list, return false
		if ( ! $this->any_product && count( $this->products ) > $matching_products_in_cart ) {
			return false;
		}

		// If we get here, matching rule is set to any product, so we can
		// use the number of matching products to determine if we have a match
		return boolval( $matching_products_in_cart );
	}

	/**
	 * @return bool
	 */
	public function is_valid_upsell(): bool {
		return $this->upsell && 'specific_products' === $this->display_for && count( $this->products );
	}

	public function is_valid(): bool {
		$product = $this->get_offer_product();

		return $product->is_purchasable() && $product->is_in_stock();
	}

	/**
	 * @param int $needle_product_id
	 * @return int
	 */
	public function quantity_in_cart( int $needle_product_id ): int {
		$needle_product = wc_get_product( $needle_product_id );

		if ( ! $needle_product ) {
			return 0;
		}

		foreach ( WC()->cart->get_cart() as $cart_item ) {
			$cart_item_variation_id = ! empty( $cart_item['variation_id'] ) ? $cart_item['variation_id'] : 0;
			$cart_item_parent_id    = $cart_item_variation_id ? wp_get_post_parent_id( $cart_item_variation_id ) : 0;
			$possible_ids           = array( $cart_item_parent_id, $cart_item_variation_id, $cart_item['product_id'] );
			$in_cart                = in_array( $needle_product_id, $possible_ids, true );

			if ( $in_cart ) {
				return $cart_item['quantity'];
			}
		}

		return 0;
	}

	/**
	 * @return int
	 */
	public function get_id(): int {
		return $this->id;
	}

	/**
	 * @return mixed
	 */
	public function get_offer_language() {
		return $this->offer_language;
	}

	/**
	 * @return mixed
	 */
	public function get_offer_description() {
		return $this->offer_description;
	}

	/**
	 * @return false|\WC_Product|null
	 */
	public function get_offer_product() {
		return wc_get_product( $this->offer_product );
	}

	/**
	 * @return string
	 */
	public function get_offer_product_price(): string {
		$product              = $this->get_offer_product();
		$price                = wc_get_price_to_display( $product, array( 'price' => $product->get_price( 'view' ) ) );
		$sale_price           = wc_get_price_to_display( $product, array( 'price' => $this->get_offer_product_sale_price() ) );
		$sale_price_formatted = wc_price( $sale_price );

		if ( $price > $sale_price ) {
			return '<del>' . wc_price( $price ) . '</del>' . $sale_price_formatted;
		}

		return $sale_price_formatted;
	}

	/**
	 * @return float|int
	 */
	public function get_offer_product_sale_price() {
		$product = $this->get_offer_product();

		$discount_type = $this->discount_type;
		$discount      = $this->offer_discount;
		$price         = $product->get_price();

		$discount_value = 'percent' === $discount_type ? ( $price / 100 ) * $discount : $discount;

		return $price - $discount_value;
	}

	public function get_display_location(): string {
		return $this->location ?? 'below_cart_items';
	}

	/**
	 * Get Displayed On Purchases Count
	 *
	 * The number of times this bump was displayed and a purchase was subsequently made.
	 *
	 * @return integer
	 */
	private function get_displayed_on_purchases_count(): int {
		return intval( get_post_meta( $this->id, 'times_bump_displayed_on_purchases', true ) );
	}

	/**
	 * Get Purchase Count
	 *
	 * The number of times this bump was added to the cart and purchased.
	 *
	 * @return integer
	 */
	private function get_purchase_count(): int {
		return intval( get_post_meta( $this->id, 'times_bump_purchased', true ) );
	}

	public function increment_displayed_on_purchases_count() {
		update_post_meta( $this->id, 'times_bump_displayed_on_purchases', $this->get_displayed_on_purchases_count() + 1 );
	}

	public function increment_purchased_count() {
		update_post_meta( $this->id, 'times_bump_purchased', $this->get_purchase_count() + 1 );
	}

	public function update_conversion_rate() {
		$purchase_count  = $this->get_purchase_count();
		$displayed_count = $this->get_displayed_on_purchases_count();
		$not_calculable  = min( $purchase_count, $displayed_count ) < 1;

		$value = $not_calculable ? 0 : round( $purchase_count / $displayed_count * 100, 2 );

		update_post_meta( $this->id, 'conversion_rate', $value );
	}

	public function get_conversion_rate() {
		$value = get_post_meta( $this->id, 'conversion_rate', true );

		return '' === $value ? '--' : floatval( $value ) . '%';
	}

	/**
	 * @return array
	 */
	public function get_products(): array {
		return (array) $this->products;
	}

	/**
	 * @throws \Exception
	 */
	static public function get( int $post_id ) {
		$self = new self();

		return $self->load( $post_id ) ? $self : false;
	}

	static public function get_post_type(): string {
		return 'cfw_order_bumps';
	}

	static public function init( $parent_menu_slug ) {
		$post_type = self::get_post_type();

		add_action(
			'init',
			function() use ( $post_type, $parent_menu_slug ) {
				register_post_type(
					$post_type,
					array(
						'labels'             => array(
							'name'               => cfw__( 'Order Bumps', 'checkout-wc' ),
							'singular_name'      => cfw__( 'Order Bump', 'checkout-wc' ),
							'add_new'            => cfw__( 'Add New', 'checkout-wc' ),
							'add_new_item'       => cfw__( 'Add New Order Bump', 'checkout-wc' ),
							'edit_item'          => cfw__( 'Edit Order Bump', 'checkout-wc' ),
							'new_item'           => cfw__( 'New Order Bump', 'checkout-wc' ),
							'view_item'          => cfw__( 'View Order Bump', 'checkout-wc' ),
							'search_items'       => cfw__( 'Find Order Bump', 'checkout-wc' ),
							'not_found'          => cfw__( 'No order bumps were found.', 'checkout-wc' ),
							'not_found_in_trash' => cfw__( 'Not found in trash', 'checkout-wc' ),
							'menu_name'          => cfw__( 'Order Bumps', 'checkout-wc' ),
						),
						'public'             => false,
						'publicly_queryable' => true,
						'show_ui'            => true,
						'show_in_menu'       => $parent_menu_slug,
						'query_var'          => false,
						'rewrite'            => false,
						'capability_type'    => 'post',
						'has_archive'        => false,
						'hierarchical'       => false,
						'supports'           => array( 'title' ),
					)
				);
			}
		);

		add_filter(
			"manage_{$post_type}_posts_columns",
			function( $columns ) {
				$date = array_pop( $columns );

				$columns['conversion_rate'] = 'Conversion Rate' . wc_help_tip( 'Conversion Rate tracks how often a bump is added to an actual completed purchase. If 20 orders are placed and a bump was displayed on 10 of those orders and the bump was purchased 5 times, the conversion rate is 50%.' );
				$columns['date']            = $date;

				return $columns;
			}
		);

		add_action(
			"manage_{$post_type}_posts_custom_column",
			function( $column, $post_id ) {
				if ( 'conversion_rate' === $column ) {
					$bump = self::get( $post_id );
					echo $bump->get_conversion_rate();
				}
				return;
			},
			10,
			2
		);
	}

	/**
	 * @return Bump[]
	 * @throws \Exception
	 */
	static public function get_all(): array {
		$posts = get_posts(
			array(
				'post_type' => self::get_post_type(),
				'number'    => -1,
			)
		);

		$bumps = array();

		foreach ( $posts as $post ) {
			$bumps[] = self::get( $post->ID );
		}

		return array_filter( $bumps );
	}
}
